@@ -45,6 +45,8 @@ class ConfigurationManager:
             self.portal_sim_name = None
             self.sim_root = None
             self.sim_conf = None
+            self.log_file = None
+            self.log_pipe_name = None
             self.config_file = None
             self.conf_file_dir = None
             self.driver_comp = None
@@ -144,7 +146,7 @@ class ConfigurationManager:
 
         # Idiot checks
         if len(self.config_file_list) == 0:
-            self.fwk.exception('Missing config file? Something is very wrong')
+            self.fwk.error('Missing config file? Something is very wrong')
             raise ValueError('Missing config file? Something is very wrong')
 
         """
@@ -190,10 +192,8 @@ class ConfigurationManager:
         self.platform_conf['USER'] = user
 
         # Grab environment variables
-        plat_keys = list(self.platform_conf.keys())
         for (k, v) in os.environ.items():
-            if k not in plat_keys \
-                    and not any(x in v for x in '{}()$'):
+            if k not in self.platform_conf and not any(x in v for x in '{}()$'):
                 self.platform_conf[k] = v
 
         mpirun_version = self.platform_conf.get('MPIRUN_VERSION', 'OpenMPI-generic')
@@ -206,7 +206,7 @@ class ConfigurationManager:
         try:
             node_alloc_mode = self.platform_conf['NODE_ALLOCATION_MODE'].upper()
             if node_alloc_mode not in ['EXCLUSIVE', 'SHARED']:
-                self.fwk.exception("bad value for NODE_ALLOCATION_MODE. expected 'EXCLUSIVE' or 'SHARED'.")
+                self.fwk.error("bad value for NODE_ALLOCATION_MODE. expected 'EXCLUSIVE' or 'SHARED'.")
                 raise ValueError("bad value for NODE_ALLOCATION_MODE. expected 'EXCLUSIVE' or 'SHARED'.")
         except Exception:
             self.fwk.exception("missing value or bad type for NODE_ALLOCATION_MODE.  expected 'EXCLUSIVE' or 'SHARED'.")
@@ -235,21 +235,20 @@ class ConfigurationManager:
 
                 # Import environment variables into config file
                 # giving precedence to config file definitions in case of duplicates
-                conf_keys = list(conf.keys())
                 for (k, v) in os.environ.items():
-                    if k not in conf_keys and not any(x in v for x in '{}()$'):
+                    if k not in conf and not any(x in v for x in '{}()$'):
                         conf[k] = v
 
                 # Allow simulation file to override platform values
                 # and then put all platform values into simulation map
                 for key in self.platform_conf:
-                    if key in conf_keys and key not in os.environ.keys():
+                    if key in conf and key not in os.environ:
                         self.platform_conf[key] = conf[key]
-                    if key not in conf_keys:
+                    if key not in conf:
                         conf[key] = self.platform_conf[key]
 
                 # Override platform value for PORTAL_URL if in simulation
-                if 'PORTAL_URL' in conf_keys:
+                if 'PORTAL_URL' in conf:
                     self.platform_conf['PORTAL_URL'] = conf['PORTAL_URL']
 
             except (IOError, SyntaxError):
@@ -269,13 +268,13 @@ class ConfigurationManager:
                 raise
 
             if sim_name in sim_name_list:
-                self.fwk.exception('Error: Duplicate SIM_NAME in configuration files')
+                self.fwk.error('Error: Duplicate SIM_NAME in configuration files')
                 sys.exit(1)
             if sim_root in sim_root_list:
-                self.fwk.exception('Error: Duplicate SIM_ROOT in configuration files')
+                self.fwk.error('Error: Duplicate SIM_ROOT in configuration files')
                 sys.exit(1)
             if log_file in log_file_list:
-                self.fwk.exception('Error: Duplicate LOG_FILE in configuration files')
+                self.fwk.error('Error: Duplicate LOG_FILE in configuration files')
                 sys.exit(1)
             if 'SIMULATION_CONFIG_FILE' not in conf:
                 conf['SIMULATION_CONFIG_FILE'] = conf_file
@@ -413,7 +412,7 @@ class ConfigurationManager:
             elif pn_simconf.upper() == 'EXCLUSIVE':
                 sim_data.sim_conf['NODE_ALLOCATION_MODE'] = 'EXCLUSIVE'
             else:
-                self.fwk.exception("Bad 'NODE_ALLOCATION_MODE' value %s" % pn_simconf)
+                self.fwk.error("Bad 'NODE_ALLOCATION_MODE' value %s" % pn_simconf)
                 raise Exception("Bad 'NODE_ALLOCATION_MODE' value %s" % pn_simconf)
         except Exception:
             sim_data.sim_conf['NODE_ALLOCATION_MODE'] = self.platform_conf['NODE_ALLOCATION_MODE']
@@ -606,21 +605,19 @@ class ConfigurationManager:
         parent_sim = self.sim_map[parent_sim_name]
         # Incorporate environment variables into config file
         # Use config file entries when duplicates are detected
-        conf_keys = list(conf.keys())
         for (k, v) in os.environ.items():
             # Do not include functions from environment
-            if k not in conf_keys and \
-                    not any(x in v for x in '{}()$'):
+            if k not in conf and not any(x in v for x in '{}()$'):
                 conf[k] = v
 
         # Allow propagation of entries from platform config file to simulation
         # config file
-        for keyword in list(self.platform_conf.keys()):
-            if keyword not in list(conf.keys()):
-                conf[keyword] = self.platform_conf[keyword]
+        for keyword, value in self.platform_conf.items():
+            if keyword not in conf:
+                conf[keyword] = value
         if override:
-            for kw in list(override.keys()):
-                conf[kw] = override[kw]
+            for kw, v in override.items():
+                conf[kw] = v
         try:
             sim_name = conf['SIM_NAME']
             sim_root = conf['SIM_ROOT']
@@ -633,10 +630,10 @@ in configuration file %s', config_file)
             self.fwk.error('Error: Duplicate SIM_NAME %s in configuration files' % (sim_name))
             raise Exception('Duplicate SIM_NAME %s in configuration files' % (sim_name))
         if sim_root in self.sim_root_list:
-            self.fwk.exception('Error: Duplicate SIM_ROOT in configuration files')
+            self.fwk.error('Error: Duplicate SIM_ROOT in configuration files')
             raise Exception('Duplicate SIM_ROOT in configuration files')
         if log_file in self.log_file_list:
-            self.fwk.exception('Error: Duplicate LOG_FILE in configuration files')
+            self.fwk.error('Error: Duplicate LOG_FILE in configuration files')
             raise Exception('Duplicate LOG_FILE in configuration files')
 
         # Add path to configuration file to simulation configuration in memory
@@ -753,4 +750,5 @@ in configuration file %s', config_file)
             raise
         for k in list(self.sim_map.keys()):
             del self.sim_map[k]
-        self.log_process.terminate()
+        if self.log_process is not None:
+            self.log_process.terminate()
