@@ -1,4 +1,4 @@
-#! /usr/bin/env python3
+#!/usr/bin/env python3
 # -------------------------------------------------------------------------------
 # Copyright 2006-2021 UT-Battelle, LLC. See LICENSE for more information.
 # -------------------------------------------------------------------------------
@@ -59,9 +59,10 @@ import socket
 import logging
 import os
 import time
+import hashlib
 from ipsframework import platformspec
 from ipsframework.messages import Message, ServiceRequestMessage, \
-    ServiceResponseMessage, MethodInvokeMessage
+    ServiceResponseMessage, MethodInvokeMessage, MethodResultMessage
 from ipsframework.configurationManager import ConfigurationManager
 from ipsframework.taskManager import TaskManager
 from ipsframework.resourceManager import ResourceManager
@@ -171,17 +172,17 @@ class Framework:
             self.log_level = logging.DEBUG
         # create handler and set level to debug
         logger.setLevel(self.log_level)
-        ch = logging.StreamHandler(self.log_file)
-        ch.setLevel(self.log_level)
+        self.ch = logging.StreamHandler(self.log_file)
+        self.ch.setLevel(self.log_level)
         # create formatter
         formatter = logging.Formatter("%(asctime)s %(name)-15s %(levelname)-8s %(message)s")
         # add formatter to ch
-        ch.setFormatter(formatter)
+        self.ch.setFormatter(formatter)
         # add ch to logger
-        logger.addHandler(ch)
+        logger.addHandler(self.ch)
         self.logger = logger
         self.verbose_debug = verbose_debug
-        self.outstanding_calls_list = []
+        self.outstanding_calls_list = {}
         self.call_queue_map = {}
 
         # add the handler to the root logger
@@ -258,7 +259,7 @@ class Framework:
                 ret_val = handler(msg)
             except BlockedMessageException as e:
                 if self.verbose_debug:
-                    self.debug('Blocked message : %s', e.__str__())
+                    self.debug('Blocked message : %s', str(e))
                 self.blocked_messages.append(msg)
                 return
             except Exception as e:
@@ -311,7 +312,7 @@ class Framework:
         """
         Produce **exception** message in simulation log file. See :func:`logging.exception` for usage.
         """
-        self.logger.exception(msg, *args, exc_info=False)
+        self.logger.exception(msg, *args)
 
     def critical(self, msg, *args):
         """
@@ -346,12 +347,11 @@ class Framework:
             self.blocked_messages = []
             for msg in msg_list:
                 self.debug('Framework processing message %s ', msg.message_id)
-                if msg.__class__.__name__ == 'ServiceRequestMessage':
+                if isinstance(msg, ServiceRequestMessage):
                     self._dispatch_service_request(msg)
                     continue
-                elif msg.__class__.__name__ == 'MethodResultMessage':
-                    self.debug('Received Result for call %s' %
-                               (msg.call_id))
+                elif isinstance(msg, MethodResultMessage):
+                    self.debug('Received Result for call %s', msg.call_id)
                     if msg.call_id not in outstanding_fwk_calls:
                         self.task_manager.return_call(msg)
                     else:
@@ -422,7 +422,7 @@ class Framework:
                 for method in ['step', 'finalize']:
                     req_msg = ServiceRequestMessage(self.component_id, self.component_id,
                                                     comp_id, 'init_call', method, 0)
-                    msg_list.append(req_msg)
+                    msg_list.append((req_msg, None, str(comp_id), method, 0))
 
                 outstanding_sim_calls[str(comp_id)] = msg_list
 
@@ -431,7 +431,7 @@ class Framework:
             #                           driver.init(), driver.step(), driver.finalize()]
             # these messages will be sent on a FIFO basis, thus running the init components,
             # then the corresponding drivers.
-            for sim_name, comp_list in list(sim_comps.items()):
+            for sim_name, comp_list in sim_comps.items():
                 msg_list = []
                 self._send_monitor_event(sim_name, 'IPS_START', 'Starting IPS Simulation')
                 self._send_dynamic_sim_event(sim_name=sim_name, event_type='IPS_START')
@@ -448,7 +448,7 @@ class Framework:
                                                         self.component_id,
                                                         comp_id,
                                                         'init_call', method, 0)
-                        msg_list.append(req_msg)
+                        msg_list.append((req_msg, sim_name, str(comp_id), method, 0))
                     # SIMYAN: add the msg_list to the outstanding sim calls
                     if msg_list:
                         outstanding_sim_calls[sim_name] = msg_list
@@ -460,12 +460,16 @@ class Framework:
 
         # send off first round of invocations...
         try:
-            for sim_name, msg_list in list(outstanding_sim_calls.items()):
-                msg = msg_list.pop(0)
+            for sim_name, msg_list in outstanding_sim_calls.items():
+                msg, sim_name, comp, method, arg = msg_list.pop(0)
                 self.debug('Framework sending message %s ', msg.__dict__)
+                if sim_name is not None:
+                    self._send_monitor_event(sim_name=sim_name,
+                                             comment=f'Target = {comp}:{method}({arg})',
+                                             eventType='IPS_CALL_BEGIN')
                 call_id = self.task_manager.init_call(msg, manage_return=False)
                 self.call_queue_map[call_id] = msg_list
-                self.outstanding_calls_list.append(call_id)
+                self.outstanding_calls_list[call_id] = sim_name, comp, method, arg, time.time()
         except Exception:
             self.exception('encountered exception during fwk.run() sending first round of invocations (init of inits and fwk comps)')
             self.terminate_all_sims(status=Message.FAILURE)
@@ -491,7 +495,7 @@ class Framework:
                     self.debug('Framework processing message %s ', msg.message_id)
 
                 sim_name = msg.sender_id.get_sim_name()
-                if msg.__class__.__name__ == 'ServiceRequestMessage':
+                if isinstance(msg, ServiceRequestMessage):
                     try:
                         self._dispatch_service_request(msg)
                     except Exception:
@@ -499,12 +503,20 @@ class Framework:
                         self.terminate_all_sims(status=Message.FAILURE)
                         return False
                     continue
-                elif msg.__class__.__name__ == 'MethodResultMessage':
+                elif isinstance(msg, MethodResultMessage):
                     if msg.call_id not in self.outstanding_calls_list:
                         self.task_manager.return_call(msg)
                         continue
                     # Message is a result from a framework invocation
-                    self.outstanding_calls_list.remove(msg.call_id)
+                    sim_name, comp, method, arg, start_time = self.outstanding_calls_list.pop(msg.call_id)
+                    if sim_name is not None:
+                        self._send_monitor_event(sim_name=sim_name,
+                                                 comment=f'Target = {comp}:{method}({arg})',
+                                                 eventType='IPS_CALL_END',
+                                                 start_time=start_time,
+                                                 end_time=time.time(),
+                                                 target=comp,
+                                                 operation=f'{method}({arg})')
                     sim_msg_list = self.call_queue_map[msg.call_id]
                     del self.call_queue_map[msg.call_id]
                     if msg.status == Message.FAILURE:
@@ -521,14 +533,18 @@ class Framework:
                         comment = 'Simulation Ended'
                         ok = True
                     try:
-                        next_call_msg = sim_msg_list.pop(0)
+                        next_call_msg, sim_name, comp, method, arg = sim_msg_list.pop(0)
+                        if sim_name is not None:
+                            self._send_monitor_event(sim_name=sim_name,
+                                                     comment=f'Target = {comp}:{method}({arg})',
+                                                     eventType='IPS_CALL_BEGIN')
                         call_id = self.task_manager.init_call(next_call_msg,
                                                               manage_return=False)
-                        self.outstanding_calls_list.append(call_id)
+                        self.outstanding_calls_list[call_id] = sim_name, comp, method, arg, time.time()
                         self.call_queue_map[call_id] = sim_msg_list
                     except IndexError:
                         sim_comps = self.config_manager.get_component_map()  # Get any new dynamic simulations
-                        if sim_name in list(sim_comps.keys()):
+                        if sim_name in sim_comps:
                             self._send_monitor_event(sim_name, 'IPS_END',
                                                      comment, ok)
                             self._send_dynamic_sim_event(sim_name, 'IPS_END', ok)
@@ -537,6 +553,7 @@ class Framework:
 
         self.terminate_all_sims(Message.SUCCESS)
         self.event_service._print_stats()
+        self.logger.removeHandler(self.ch)
         return True
 
     def initiate_new_simulation(self, sim_name):
@@ -554,16 +571,16 @@ class Framework:
                 req_msg = ServiceRequestMessage(self.component_id,
                                                 self.component_id, comp_id,
                                                 'init_call', method, 0)
-                msg_list.append(req_msg)
+                msg_list.append((req_msg, sim_name, str(comp_id), method, 0))
 
         # send off first round of invocations...
-        msg = msg_list.pop(0)
+        msg, sim_name, comp, method, arg = msg_list.pop(0)
         self.debug('Framework sending message %s ', msg.__dict__)
         call_id = self.task_manager.init_call(msg, manage_return=False)
         self.call_queue_map[call_id] = msg_list
-        self.outstanding_calls_list.append(call_id)
+        self.outstanding_calls_list[call_id] = sim_name, comp, method, arg, time.time()
 
-    def _send_monitor_event(self, sim_name='', eventType='', comment='', ok='True'):
+    def _send_monitor_event(self, sim_name='', eventType='', comment='', ok=True, target=None, operation=None, start_time=None, end_time=None):
         """
         Publish a portal monitor event to the *_IPS_MONITOR* event topic.
         Event topics that start with an underscore are reserved for use by the
@@ -633,6 +650,27 @@ class Framework:
             portal_data['state'] = 'Completed'
             portal_data['stopat'] = time.strftime('%Y-%m-%d|%H:%M:%S%Z',
                                                   time.localtime())
+            # Zipkin json format
+            portal_data['trace'] = {"timestamp": int(self.start_time*1e6),
+                                    "duration": int((time.time() - self.start_time)*1e6),
+                                    "localEndpoint": {
+                                        "serviceName": str(self.component_id)
+                                    },
+                                    "id": hashlib.md5(str(self.component_id).encode()).hexdigest()[:16],
+                                    'tags': {'total_cores': str(self.resource_manager.total_cores)}}
+        elif eventType == "IPS_CALL_END":
+            trace = {}  # Zipkin json format
+            if start_time is not None and end_time is not None:
+                trace['timestamp'] = int(start_time*1e6)  # convert to microsecond
+                trace['duration'] = int((end_time-start_time)*1e6)  # convert to microsecond
+            if target is not None:
+                trace['localEndpoint'] = {"serviceName": target}
+                trace['name'] = operation
+                trace['id'] = hashlib.md5(f"{target}:{operation}".encode()).hexdigest()[:16]
+                trace["parentId"] = hashlib.md5(str(self.component_id).encode()).hexdigest()[:16]
+
+            if trace:
+                portal_data['trace'] = trace
 
         event_body = {}
         event_body['sim_name'] = sim_name
