@@ -84,7 +84,6 @@ class PortalBridge(Component):
         self.host = ''
         self.curTime = time.localtime()
         self.startTime = self.curTime
-        self.services = services
         self.sim_map = {}
         self.portal_url = None
         self.done = False
@@ -199,6 +198,9 @@ class PortalBridge(Component):
         portal_data['portal_runid'] = sim_data.portal_runid
         portal_data['seqnum'] = sim_data.counter
 
+        if 'trace' in portal_data:
+            portal_data['trace']['traceId'] = hashlib.md5(sim_data.portal_runid.encode()).hexdigest()
+
         self.send_event(sim_data, portal_data)
         sim_data.counter += 1
         self.counter += 1
@@ -253,7 +255,7 @@ class PortalBridge(Component):
             try:
                 if self.first_event:  # First time, launch sendPost.py daemon
                     cmd = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'sendPost.py')
-                    python_exec = shutil.which('python')
+                    python_exec = shutil.which('python3')
                     self.childProcess = Popen([python_exec, cmd], bufsize=128,
                                               stdin=PIPE, stdout=PIPE,
                                               stderr=PIPE, close_fds=True)
