@@ -223,12 +223,13 @@ class TaskManager:
         wnodes = init_task_msg.args[5]
         wsocks = init_task_msg.args[6]
         tcpp = init_task_msg.args[7]
+        omp = init_task_msg.args[8]
 
         # SIMYAN: increased arguments
-        cmd_args = init_task_msg.args[8:]
+        cmd_args = init_task_msg.args[9:]
 
         try:
-            return self._init_task(caller_id, nproc, binary, working_dir, tppn, tcpp, wnodes, wsocks, cmd_args)
+            return self._init_task(caller_id, nproc, binary, working_dir, tppn, tcpp, omp, wnodes, wsocks, cmd_args)
         except InsufficientResourcesException:
             if block:
                 raise BlockedMessageException(init_task_msg, '***%s waiting for %d resources' %
@@ -246,7 +247,7 @@ class TaskManager:
         except Exception:
             raise
 
-    def _init_task(self, caller_id, nproc, binary, working_dir, tppn, tcpp, wnodes, wsocks, cmd_args):
+    def _init_task(self, caller_id, nproc, binary, working_dir, tppn, tcpp, omp, wnodes, wsocks, cmd_args):
         # handle for task related things
         task_id = self.get_task_id()
 
@@ -260,9 +261,9 @@ class TaskManager:
         self.fwk.debug('RM: get_allocation() returned %s', str(retval))
         partial_node = retval[0]
         if partial_node:
-            (nodelist, corelist, ppn, max_ppn, accurateNodes) = retval[1:]
+            (nodelist, corelist, ppn, max_ppn, accurateNodes, cores_allocated) = retval[1:]
         else:
-            (nodelist, ppn, max_ppn, cpp, accurateNodes) = retval[1:]
+            (nodelist, ppn, max_ppn, cpp, accurateNodes, cores_allocated) = retval[1:]
 
         if partial_node:
             nodes = ','.join(nodelist)
@@ -282,7 +283,7 @@ class TaskManager:
                                                       max_ppn, nodes,
                                                       accurateNodes,
                                                       False, task_id,
-                                                      cpp)
+                                                      cpp, omp)
 
         self.curr_task_table[task_id] = {'component': caller_id,
                                          'status': 'init_task',
@@ -292,11 +293,11 @@ class TaskManager:
                                          'launch_cmd': cmd,
                                          'env_update': env_update}
 
-        return (task_id, cmd, env_update)
+        return (task_id, cmd, env_update, cores_allocated)
 
     def build_launch_cmd(self, nproc, binary, cmd_args, working_dir, ppn,
                          max_ppn, nodes, accurateNodes, partial_nodes,
-                         task_id, cpp=0, core_list=''):
+                         task_id, cpp=0, omp=False, core_list=''):
         """
         Construct task launch command to be executed by the component.
 
@@ -519,9 +520,10 @@ class TaskManager:
                                 nproc_flag, str(nproc),
                                 cpuptask_flag, str(cpp),
                                 cpubind_flag])
-                env_update = {'OMP_PLACES': 'threads',
-                              'OMP_PROC_BIND': 'spread',
-                              'OMP_NUM_THREADS': str(cpp)}
+                if omp:
+                    env_update = {'OMP_PLACES': 'threads',
+                                  'OMP_PROC_BIND': 'spread',
+                                  'OMP_NUM_THREADS': str(cpp)}
         else:
             self.fwk.error("invalid task launch command.")
             raise RuntimeError("invalid task launch command.")
@@ -548,23 +550,23 @@ class TaskManager:
         ret_dict = {}
         for task_name in task_dict:
             # handle for task related things
-            (nproc, working_dir, binary, cmd_args, tppn, wnodes, wsocks, tcpp) = task_dict[task_name]
+            (nproc, working_dir, binary, cmd_args, tppn, wnodes, wsocks, tcpp, omp) = task_dict[task_name]
 
             try:
-                ret_dict[task_name] = self._init_task(caller_id, nproc, binary, working_dir, tppn, tcpp, wnodes, wsocks, cmd_args)
+                ret_dict[task_name] = self._init_task(caller_id, nproc, binary, working_dir, tppn, tcpp, omp, wnodes, wsocks, cmd_args)
             except InsufficientResourcesException:
                 continue
             except BadResourceRequestException as e:
                 self.fwk.error("There has been a fatal error, %s requested %d too many processors in task %d",
                                caller_id, e.deficit, e.task_id)
-                for task_id, _, _ in ret_dict.values():
+                for task_id, _, _, _ in ret_dict.values():
                     self.resource_mgr.release_allocation(task_id, -1)
                     del self.curr_task_table[task_id]
                 raise
             except ResourceRequestMismatchException as e:
                 self.fwk.error("There has been a fatal error, %s requested too few processors per node to launch task %d (request: procs = %d, ppn = %d)",
                                caller_id, e.task_id, e.nproc, e.ppn)
-                for task_id, _, _ in ret_dict.values():
+                for task_id, _, _, _ in ret_dict.values():
                     self.resource_mgr.release_allocation(task_id, -1)
                     del self.curr_task_table[task_id]
                 raise
